*DECK D2PINP
      SUBROUTINE D2PINP(  IPSAP, IPDAT ,  IPHEL, IPINP,  FILNAM, IPRINT,
     >                   STAVEC, CRDINF,   NCRD,   PKEY,  SFAC,    BFAC,
     >                     ISOT,   MESH, USRPAR, USRVAL, USRSTA,USRVAPK,
     >                      SAP,    MIC, EXC   ,   SCAT, ADF   ,    DEB,
     >                      COM,   FA_K,   IUPS,   LADD,   LNEW,    MIX,
     >                      FC1,    FC2,    FC3,    FC4,    XSC,    NGP,
     >                   JOBTIT, JOBOPT,    DER,   VERS, SIGNAT, MIXDIR,
     >                      CDF,    GFF,   ADFD,   CDFD,    YLD, YLDOPT,
     >                   LOCYLD,   XESM                                )
*
*-----------------------------------------------------------------------
*
*Purpose:
* 1) Recover data from saphyb or multicompo object.
* 2) Build headers of GenPMAXS and Helios like file
*
*Copyright:
* Copyright (C) 2015 IRSN
*
*Author(s): J. Taforeau
*
*parameters: input/output
* IPSAP   address of saphyb or multicompo object
* IPDAT   address of data structure INFO
* IPHEL   file unit of Helios like file
* IPINP   file unit of GenPMAXS input file
* NGP     number of energy groups recovered from D2P input user
* NCRD    number of control rod composition recovered from D2P
*         input user
* MIX     index of mixture on which XS are to be extracted (only
*         for reflector cases)
* FA_K    assembly type
*         =0 reflector
*         =1 assembly
* IUPS    treatment for upscattering
*         =0 keep up scatter XS
*         =1 remove up scatter XS, modify  down scatter with DRAGON
*         spectrum (not available in this version)
*         =2 remove up scatter XS, modify  down scatter with infinite
*         medium spectrum
* USRSTA  state variable names recovered from GLOBAL record in D2P:
* USRVAL  number of value for state variables  recovered from GLOBAL
*         record in D2P:
* IPRINT  control the printing on screen
* STAVEC  various parameters associated with the IPDAT structure
* CRDINF  meaning of control rods in the IPSAP object
* VERS    version of PARCS to be used
* SFAC    the scattering cross section factor
* BFAC    the multiplier for betas
* FC1     FILE_CONT_1 recovered from D2P: input
* FC2     FILE_CONT_2 recovered from D2P: input
* FC3     FILE_CONT_3 recovered from D2P: input
* FC4     FILE_CONT_4 recovered from D2P: input
* XSC     XS_CONT     recovered from D2P: input
* DEB     FLAG to indicate the first call to the D2PGEN subroutine
* USRVAPK value of state prameter set by the user and recoverd from
*         USER ADD option in D2P:
* ADF     type of ADF to be selected
* DER     partials derivative (T) or row cross section (F) to be stored
*         in PMAXS
* JOBOPT  flag for JOB_OPT record in IPINP object
* USRPAR  name of state variables (sapnam) in IPSAP associated to
*         DMOD TCOM etc. recovered from PKEY card in D2P:
* MESH    type of meshing to be applied for the branching calculation
* PKEY    name of state variable (refnam) recovered from PKEY card in
*         D2P:
* FILNAM  name of IPINP
* ISOT    name of isotopes in IPSAP for xenon samarium and spomethium
* JOBTIT  title of in header of PMAXS file
* COM     comment to be printed in PMAXS file
* SAP     flag to indicate that absorption cross section must be
*         directly recovered from IPSAP
* MIC     flag to indicate that absorption cross section must be
*         directly recovered from IPMIC
* EXC     flag to indicate that excess cross section is to be extracted
*         from absoption xs (only if SAP)
* SCAT    flag to indicate that scattering cross section must be
*         directly reconstructed from IPSAP
* LADD    flag to indicate that new points must be added to the IPSAP
*         original meshing
* LNEW    flag to indicate that only new points must be used during the
*         branching calculation
* SIGNAT  signature of the object containing cross sections
* MIXDIR  directory that contains homogeneous mixture information
* CDF     type of CDF to be selected
* GFF     type of GFF to be selected
* ADFD    name of record for 'DRA' type of ADF
* CDFD    name of record for 'DRA' type of CDF
* YLD     user defined values for fission yields (1:I, 2:XE, 3:PM)
* LOCYLD  value for state parameter on which fission yield will be
*         calculated
* YLDOPT  option for fission yields calculation (DEF, MAN, FIX)
* XESM    option for comparing k-inf in GenPMAX (1: using Pm/Sm data;
*         2: using I/Xe data; 3: using I/Xe/Pm/Sm data)
*-----------------------------------------------------------------------
*
      USE GANLIB
*----
*  SUBROUTINE ARGUMENTS
*----
      TYPE(C_PTR) IPSAP,IPDAT
      INTEGER NGP,NCRD,MIX,FA_K,IUPS,USRSTA
      INTEGER IPHEL,IPINP,IPRINT,DEB,XESM
      INTEGER STAVEC(40),CRDINF(20),USRVAL(12)
      REAL VERS,SFAC,BFAC,YLD(3),LOCYLD(5)
      REAL FC1(5),FC2(8),FC3(7),FC4(3),XSC(3)
      REAL USRVAPK(12,10)
      CHARACTER JOBOPT(16)
      CHARACTER*3 ADF,CDF,GFF,YLDOPT
      CHARACTER*8 ADFD(4),CDFD(8)
      CHARACTER*4 DER
      CHARACTER*5 MESH
      CHARACTER*8 PKEY(6)
      CHARACTER*12 FILNAM,ISOT(4), SIGNAT,MIXDIR,USRPAR(12)
      CHARACTER*16 JOBTIT
      CHARACTER*40 COM
      LOGICAL SAP, MIC, EXC,SCAT,LADD,LNEW
*----
*  LOCAL VARIABLES
*----
      LOGICAL LADF,LCDF,LGFF,LYLD
      INTEGER NADF,NCDF
      
      IF (JOBOPT(1)=='T') THEN
        NADF=STAVEC(13)
        IF (NADF.NE.XSC(1)) THEN
         WRITE(6,*)'@D2PINP: INCOHERENT NUMBER OF ADF (',NADF,
     >   ')','AND NUMBER OF SIDES IN ASSEMBLY (',XSC(1),').'
         CALL XABORT ("=> CHECK CARD 'ADF' AND 'XS_CONT'")     
        ENDIF
        IF ((SIGNAT.EQ.'L_SAPHYB').and.(ADF.EQ.'DRA')) THEN
          WRITE(6,*) "@D2PINP: ADF OF TYPE (",ADF,
     1     ") NOT YET IMPLEMENTED WITH SAPHYB OBJECT"
          WRITE(6,*)"=> WARNING :  ADF CALUCLATION IGNORED"
          LADF = .FALSE.
          JOBOPT(1)='F'   
        ELSE IF ((SIGNAT.EQ.'L_MULTICOMPO').and.(ADF.NE.'DRA')) THEN    
          WRITE(6,*) "@D2PINP: ADF OF TYPE (",ADF,
     1     " NOT YET IMPLEMENTED WITH MULTICOMPO OBJECT"
          WRITE(6,*)"=> WARNING :  ADF CALUCLATION IGNORED"
          LADF = .FALSE.
          JOBOPT(1)='F'          
        ELSE
          LADF = .TRUE.
        ENDIF
      ELSE
        LADF = .FALSE.
      ENDIF
      IF (JOBOPT(10)=='T') THEN
        NCDF=STAVEC(15)
        IF (NCDF.NE.XSC(2)) THEN
         WRITE(6,*)'@D2PINP: INCOHERENT NUMBER OF CDF (',NCDF,
     >   ')','AND NUMBER OF CORNERS IN ASSEMBLY (',XSC(2),').'
         CALL XABORT ("=> CHECK CARD 'CDF' AND 'XS_CONT'")     
        ENDIF
        IF (SIGNAT.EQ.'L_SAPHYB') THEN
          WRITE(6,*) "@D2PINP: CDF CALCULATION",
     1     " NOT YET IMPLEMENTED WITH SAPHYB OBJECT"
          WRITE(6,*)"=> WARNING :  CDF CALUCLATION IGNORED"        
          LCDF = .FALSE.     
          JOBOPT(10)='F'           
        ENDIF
        IF (CDF.NE. 'DRA') THEN
         CALL XABORT ("@D2PINP UNKNOW CDF TYPE : "//CDF//'.')
        ENDIF
        LCDF = .TRUE.
      ELSE
        LCDF = .FALSE.
      ENDIF
      IF (JOBOPT(11)=='T') THEN
        IF (SIGNAT.EQ.'L_SAPHYB') THEN
          WRITE(6,*) "@D2PINP: GFF CALCULATION",
     1     " NOT YET IMPLEMENTED WITH SAPHYB OBJECT"
          WRITE(6,*)"=> WARNING :  GFF CALUCLATION IGNORED"        
          LGFF = .FALSE.     
          JOBOPT(11)='F'           
        ENDIF
        IF (GFF.NE. 'DRA') THEN
         CALL XABORT ("@D2PINP UNKNOW GFF TYPE : '"//GFF//"'.")
        ENDIF
        LGFF = .TRUE.
      ELSE
        LGFF = .FALSE.
      ENDIF
      
      IF (JOBOPT(9)=='T') LYLD = .TRUE.
      IF ((JOBOPT(2)=='T').and.(JOBOPT(9)=='F')) THEN
          WRITE(6,*) "@D2PINP: JOB_OPT : XE/SM ARE REQUESTED (lxes=T) ",
     1     "BUT FISSION YIELDS ARE NOT RECOVERED (lyld=F) "
         CALL XABORT("=> PLEASE TURN ON THE lyld FLAG ")
      ENDIF
         
      IF((FA_K.EQ.1).OR.(FA_K.EQ.0)) THEN
*       CASE FOR FUEL PMAXS
        IF (SIGNAT.EQ.'L_SAPHYB') THEN
        
        WRITE(6,*) "*******    EXTRACTION OF DATA FROM SAPHYB      ****"
        CALL    D2PSAP (  IPSAP,  IPDAT, STAVEC, CRDINF,   NCRD,   PKEY,
     >                    ISOT ,   MESH, USRPAR, USRVAL, USRSTA,USRVAPK,
     >                    SAP  ,    MIC,    EXC,   SCAT,    ADF,   LADD,
     >                    LNEW ,   LADF, IPRINT,   LYLD,    YLD, YLDOPT,
     >                    LOCYLD                                       )
     
        ELSE  IF (SIGNAT.EQ.'L_MULTICOMPO') THEN
        
        WRITE(6,*) "*******    EXTRACTION OF DATA FROM MULTICOMPO  ****"
        WRITE(6,*)
        WRITE(6,*) "DIRECTORY:'",MIXDIR,"' AT MIXUTRE INDEX ",MIX,"." 
        WRITE(6,*) "=> WARNING: CHECK EXISTENCE OF ",MIXDIR,"DIRECTORY."

         IF (LADF) THEN
          WRITE(6,*) "ADF CALCULATION REQUESTED:"
          WRITE(6,*)"=> WARNING: CHECK EXISTENCE OF '",ADFD(1:NADF),
     >     "' RECORDS"
         ENDIF
         IF (LCDF) THEN
          WRITE(6,*) "CDF CALCULATION REQUESTED:"
          WRITE(6,*)"=> WARNING: CHECK EXISTENCE OF '",CDFD(1:NCDF),
     >     "' RECORDS"
         ENDIF

        CALL LCMLIB(IPSAP)

        CALL  D2PMCO   (  IPSAP,  IPDAT, STAVEC, CRDINF,   NCRD,   PKEY,
     >                    ISOT ,   MESH, USRPAR, USRVAL, USRSTA,USRVAPK,
     >                    SAP  ,    MIC,    EXC,   SCAT,    ADF,   LADD,
     >                    LNEW ,   LADF, IPRINT, MIXDIR,    MIX,   LCDF,
     >                    LGFF ,    CDF,    GFF,   ADFD,   CDFD,  LYLD ,
     >                      YLD, YLDOPT, LOCYLD                        )
        ELSE
         CALL XABORT ('@D2PINP: UNKNOWN SIGNATURE')
        ENDIF
      ELSE  
        CALL XABORT('@D2PINP: PHASE 1: FUEL OR REFLECTOR CARD EXPECTED')
      ENDIF

      WRITE(6,*) "*******    CREATION OF HELIOS LIKE FILE         ****"
      IF (YLDOPT.EQ.'MAN') THEN
       DEB = -1
      ELSE
       DEB = 0
      ENDIF
      CALL      D2PHEL ( IPHEL,  IPDAT,  STAVEC,    FC1,    FC2,    NGP,
     >                     FC3,    FC4,     XSC, IPRINT                )

      IF(IPRINT > 0)  THEN
       WRITE(6,*) "*******    CREATION OF GENPMAXS INPUT FILE      ****"
      ENDIF

      CALL      D2PGEN ( IPINP, IPDAT,   STAVEC, JOBTIT, FILNAM,    DER,
     >                    VERS,   COM,   JOBOPT,   IUPS,   FA_K,   SFAC,
     >                    BFAC,   DEB,     XESM,  IPRINT               )

      CALL LCMSIX(IPDAT,' ',0)
      CALL LCMPUT(IPDAT,'STATE-VECTOR',40,1,STAVEC)

      END
